package com.puppycrawl.tools.checkstyle.checks.naming;

import java.io.ObjectStreamField;

public class InputConstantNames
{
    private static final long serialVersionUID = 1L; //should be ignored
    private static final ObjectStreamField[] serialPersistentFields = {}; // should be ignored too
}
