package com.google.checkstyle.test.chapter3filestructure.rule333orderingandspacing;

import static java.io.File.createTempFile;
import static java.awt.Button.ABORT; //warn
import static javax.swing.WindowConstants.*;


import java.util.StringTokenizer; //warn
import java.util.*; //warn
import java.util.concurrent.AbstractExecutorService; //warn
import java.util.concurrent.*; //warn

import com.google.checkstyle.test.chapter2filebasic.rule21filename.*;
import com.sun.xml.internal.xsom.impl.scd.Iterators; //warn

import com.google.common.reflect.*; //warn

public class InputCustomImportOrder3 {
}
