package com.puppycrawl.tools.checkstyle.checks.javadoc;

public class InputInheritDoc
{
    /** {@inheritDoc} */
    InputInheritDoc()
    {
    }

    /** {@inheritDoc} */
    private void privateMethod()
    {
    }

    /** {@inheritDoc} */
    void packageMethod()
    {
    }

    /** {@inheritDoc} */
    protected void protectedMethod()
    {
    }

    /** {@inheritDoc} */
    public void publicMethod()
    {
    }
    
    /** {@inheritDoc} */
    private static void privateStaticMethod()
    {
    }

    /** {@inheritDoc} */
    static void packageStaticMethod()
    {
    }

    /** {@inheritDoc} */
    protected static void protectedStaticMethod()
    {
    }

    /** {@inheritDoc} */
    public static void publicStaticMethod()
    {
    }
}
