package com.puppycrawl.tools.checkstyle.checks.naming;

public class InputMemberName
{
    public int mPublic;
    protected int mProtected;
    int mPackage;//comment
    private int mPrivate;

    public int _public;
    protected int _protected;
    int _package;
    private int _private;
}
