//Compilable by javac, but noncompilable by eclipse

//////////////////////////////////////////
//Some header
//////////////////////////////////////////
import static java.io.File.createTempFile;
import static java.awt.Button.ABORT;
import static javax.swing.WindowConstants.*;

import java.awt.Button;
import java.awt.Frame;
import java.awt.Dialog;
import java.awt.event.ActionEvent;
import javax.swing.JComponent;
import javax.swing.JTable;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;

import com.puppycrawl.tools.*;

import com.google.common.*;
import org.apache.*;

public class InputImportOrder {
}