package com.puppycrawl.tools.checkstyle.doclets;

import com.puppycrawl.tools.checkstyle.grammars.GeneratedJavaTokenTypes;

public class InputTokenTypesDocletCorrect {

        /**
         * The end of file token.  This is the root node for the source
         * file.  It's children are an optional package definition, zero
         * or more import statements, and one or more class or interface
         * definitions.
         **/
        public static final int EOF = GeneratedJavaTokenTypes.EOF;
}
