package com.google.checkstyle.test.chapter7javadoc.rule713atclauses;

class InputNonEmptyAtclauseDescriptionCheckSpaceSeq
{
        /**
         * Some javadoc.
         * @param a Some javadoc.
         * @param b Some javadoc.
         */
        public InputNonEmptyAtclauseDescriptionCheckSpaceSeq(String a, int b)
        {
                
        }
        
        /**
         * Some javadoc.
         * @param a Some javadoc.
         * @deprecated Some javadoc.
         */
        public InputNonEmptyAtclauseDescriptionCheckSpaceSeq(String a)
        {
                
        }
        
        /**
         * Some javadoc.
         * @param a                     
         * @param b           
         * @param c     
         */
        public InputNonEmptyAtclauseDescriptionCheckSpaceSeq(String a, int b, double c)
        {
                
        }
        
        /**
         * 
         * @param a  
         * @param e     
         * @deprecated     
         */
        public InputNonEmptyAtclauseDescriptionCheckSpaceSeq(String a, boolean e)
        {
                
        }
}
