package com.google.checkstyle.test.chapter3filestructure.rule331nowildcard;

import java.io.*; //warn
import java.lang.*; //warn
import java.lang.String;
import java.sql.Connection;
import java.util.List;
import java.util.List;
import java.util.Iterator;
import java.util.Enumeration;
import java.util.Arrays;
import javax.swing.JToolBar;
import javax.swing.JToggleButton;
import javax.swing.ScrollPaneLayout;
import javax.swing.BorderFactory;
import static java.io.File.listRoots;

import static javax.swing.WindowConstants.*; //warn
import static javax.swing.WindowConstants.*; //warn
import static java.io.File.createTempFile;
import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.HeadlessException;
import java.awt.Label;
import java.util.Date;
import java.util.Calendar;
import java.util.BitSet;

class InputAvoidStarImport {}
