package com.google.checkstyle.test.chapter3filestructure.rule333orderingandspacing;

import static java.io.File.createTempFile;
import static java.awt.Button.ABORT; //warn
import static javax.swing.WindowConstants.*;

import java.awt.Button; //warn
import java.awt.Frame; //warn
import java.awt.Dialog; //warn
import java.awt.event.ActionEvent; //warn
import javax.swing.JComponent; //warn
import javax.swing.JTable; //warn
import java.io.File; //warn
import java.io.IOException; //warn
import java.io.InputStream; //warn
import java.io.Reader; //warn



import com.google.common.base.Ascii;

public class InputCustomImportOrder1 {}
