package com.google.checkstyle.test. //warn
              chapter3filestructure.rule332nolinewrap;

import com.google.common.annotations.Beta; //ok

import javax.accessibility. //warn
    AccessibleAttributeSequence;
import java.util.concurrent.atomic.AtomicIntegerFieldUpdater; //ok

public class
    InputNoLineWrapBad {
    
    public void
        fooMethod() {
        final int
            foo = 0;
    }
}

enum
    FooFoo {
}

interface
    InterFoo {}


