package com.puppycrawl.tools.checkstyle.checks.imports;


import org.junit.*;

import java.*;
import javax.swing.*;

import static sun.tools.util.CommandLine.parse;
import static sun.tools.util.ModifierFilter.ALL_ACCESS;
import static sun.tools.util.ModifierFilter.ALL_ACCESS;

public class InputCustomImportOrderThirdPartyPackage
{
    
}
