h1 {
  text-transform: capitalize;
}

h2 {
  text-transform: capitalize;
}

h3 {
  text-transform: capitalize;
}

h4 {
  text-transform: capitalize;
}

.tip {
  margin-left : 5%;
  margin-right : 5%;
  padding-left: 2%;
  padding-right: 2%;
  background-color: Silver;
  color:Black;
  border: ridge;
}

#bannerLeft img {
  float:left;
  position: relative;
  top: 0;
  left: 40px;

}

#banner {
  border-bottom: 1px solid #fff;
  background: url("../images/header-background.png") repeat scroll 0 0 transparent;
}

.dl-link {
  position: relative;
  float: right;
}

h2:hover .anchor {
  display: inline-block;
}

.anchor {
  display: none;
}
