package com.puppycrawl.tools.checkstyle.checks.imports;

import static com.puppycrawl.tools.checkstyle.checks.imports.ImportOrderCheck.MSG_ORDERING;
import static java.awt.Button.ABORT;
import static java.io.File.createTempFile;
import static javax.swing.WindowConstants.*;
import static org.junit.Assert.assertEquals;

import java.awt.Button;
import java.awt.Dialog;
import java.io.InputStream;

import javax.swing.JComponent;
import javax.swing.JTable;

import org.junit.Test;
import org.powermock.api.mockito.PowerMockito;

import com.puppycrawl.tools.checkstyle.api.DetailAST;

import sun.tools.java.ArrayType;

public class InputImportOrder_EclipseDefaultPositive {
}
